using System;
using System.Threading;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
/// we must state the namespace for the CsIow-Library here
using org.wayoda.csiow;

/// create our own namespace, do not use org.wayoda.csiow !
namespace CsDemo {
    /// <summary>
    /// This is a delegate that connects this form to the 
    /// WritePanel onWriteBtnClick event
    /// </summary>
    /// <param name="rep">
    /// The report to be written. The device is the currently
    /// selected IOWarrior in the deviceListBox
    /// </param>
    public delegate void writeReportDelegate(IowReport rep);
 
    /// <summary>
    /// Our class implements the interface IowDeviceChangeListner that
    /// updates the listbox for devices.
    /// It implements the Interface IowReportListener, because ALL
    /// reports from ALL devices will simply be printed to the ReportTextBox.
    /// </summary>
    public partial class Form1 : Form, IowDeviceChangeListener, IowReportListener {
        /// <summary>
        /// Used to add an IOWarrior to a listbox with ListBox.Invoke
        /// </summary>
        /// <param name="iow">
        /// The IOWarrior to be added to the listbox
        /// </param>
        /// <returns></returns>
        delegate int AddIowToControl(IOWarrior iow);
        /// <summary>
        /// Used to remove an unplugged IOWarrior from the listbox
        /// </summary>
        /// <param name="i">
        /// The index of the unplugged IOWarrior in the listbox
        /// </param>
        delegate void RemoveIowFromControl(int i);
        /// <summary>
        /// Used to add a string to the reportTextBox with Invoke
        /// </summary>
        /// <param name="s">
        /// The string to be added.
        /// </param>
        delegate void AddReportToControl(String s);

        delegate void MoveCursorDelegate();

        
        public Form1() {
            InitializeComponent();
            //register our main form to the writebutton click event
            writePanel1.wdelegate+=this.writeToIow;
        }

        /// <summary>
        /// This is a good place to open the CsIow-Library
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e) {
            //Create a new devicefilter
            IowDeviceFilter f=new IowDeviceFilter();
            //uncomment the next line if you want to see only IOWarrior24's
            //f.addProduct(IOWarrior.PID_IOW24);
            //open our library
            IowError err;
            err=IowManager.open(this,f);
            if(err!=IowError.OK) {
                //we somehow failed to open the library
                //show a message and exit
                StringBuilder sb=new StringBuilder();
                sb.AppendLine("Opening the CsIow-Lib failed");
                sb.AppendLine("("+err.ToString()+")");
                MessageBox.Show(sb.ToString(), "SimpleIow Failed!");
                Application.Exit();
            }
            //Now we simply wait for Devices and Reports        
        }

        /// <summary>
        /// We have to close the CsIow-Library when the application exits
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
            // We have to close the library. Threads have to be stopped and
            // resources must be freed.
            IowManager.close();
        }

        /// <summary>
        /// This is a callback from the library : 
        /// a new IOWarrior was plugged in.
        /// </summary>
        /// <param name="iow">
        /// The IOWarrior just plugged in.
        /// </param>
        public void deviceAdd(IOWarrior iow) {
            //This call will NEVER be from the thread where the controls
            //in this app were created. We will have to use a delegate
            //and Invoke to add our new IOWarrior to as Listbox
            AddIowToControl ais=new AddIowToControl(deviceListBox.Items.Add);    
            deviceListBox.Invoke(ais, new Object [] {iow});
            //We want to be notified for all reports by all devices
            //so we register ourself to the IOWarrior that just came in.
            iow.addReportListener(this, null);
        }

        /// <summary>
        /// There are 2 sources for this callback. It could be the IowManger or
        /// the IOWarrior itself we registered with. For each IOWarrior that gets unplugged
        /// we are called twice. But since we simply remove the IOWarrior from the
        /// Listbox this doesn't matter much.
        /// </summary>
        /// <param name="iow">The IOWarrior that was unplugged</param>
        public void deviceRemoved(IOWarrior iow) {
            // We must use a delegate and Invoke to remove the IOWarrior
            // from the listbox.
            RemoveIowFromControl ris=new RemoveIowFromControl(deviceListBox.Items.RemoveAt);
            if(iow == null)
                return;
            for(int i=0;i<deviceListBox.Items.Count;i++) {
                // The test for the device can be done without 
                // Invoke, because the visual appearance of the
                // control is unchanged
                if(iow.Equals(deviceListBox.Items[i])) {
                    //Here we need to use Invoke for device removal
                    deviceListBox.Invoke(ris, new object[] { i });
                }
            }
        }

        /// <summary>
        /// Our reports get printed on a TextBox and we always want to scroll
        /// to the end of the reports automaticly. Since reports come
        /// from a different thread this has to be called from a delegate
        /// through Invoke. 
        /// </summary>
        private void scrollReportAreaToEnd() {
            ReportList.SelectionStart=ReportList.TextLength;
            ReportList.ScrollToCaret();
        }

        /// <summary>
        /// One of the IOWarriors we registered with send some new data
        /// in this callback.
        /// </summary>
        /// <param name="rep">
        /// The new data from the device
        /// </param>
        /// <param name="iow">
        /// The IOWarrior that send the data.
        /// </param>
        public void reportUpdate(IowReport rep, IOWarrior iow) {
            // Create a nice message to be printed in the ReportTextBox
            int reportLength;
            String dev=iow.ToString();
            StringBuilder sb=new StringBuilder();
            sb.Append("R "); 
            sb.Append(iow.getProductName());
            sb.Append(" ");
            sb.AppendFormat("{0:X} ", iow.getSerialNumber());
            if(rep.getSpecialMode()==IowSpecialMode.IO)
                reportLength=iow.getReportSizeIO();
            else
                reportLength=iow.getReportSizeSM();
            for(int i=0;i<reportLength;i++) {
                sb.AppendFormat("{0:X2} ", rep[i]);
            }
            sb.Append("\n");
            // And now add this text using a delegate and Invoke again.
            AddReportToControl adds=new AddReportToControl(ReportList.AppendText);
            ReportList.Invoke(adds, new object[] { sb.ToString() });
            // Move the cursor on the TextBox to the last line
            MoveCursorDelegate mcd=new MoveCursorDelegate(this.scrollReportAreaToEnd);
            Invoke(mcd);
            
       }

        /// <summary>
        /// This one is simply called from the WritePanel when new data was written
        /// to a device.
        /// </summary>
        /// <param name="rep">
        /// The data that was written
        /// </param>
        /// <param name="iow">
        /// The IOWarrior to which it was written
        /// </param>
        private void writeUpdate(IowReport rep, IOWarrior iow) {
            int reportLength;
            String dev=iow.ToString();
            StringBuilder sb=new StringBuilder();
            sb.Append("W ");
            sb.Append(iow.getProductName());
            sb.Append(" ");
            sb.AppendFormat("{0:X} ", iow.getSerialNumber());
            if(rep.getSpecialMode()==IowSpecialMode.IO)
                reportLength=iow.getReportSizeIO();
            else
                reportLength=iow.getReportSizeSM();
            for(int i=0;i<reportLength;i++) {
                sb.AppendFormat("{0:X2} ", rep[i]);
            }
            sb.Append("\n");
            AddReportToControl adds=new AddReportToControl(ReportList.AppendText);
            ReportList.Invoke(adds, new object[] { sb.ToString() });
            // Move the cursor on the TextBox to the last line
            MoveCursorDelegate mcd=new MoveCursorDelegate(this.scrollReportAreaToEnd);
            Invoke(mcd);
        }

        /// <summary>
        /// Enable or disable the WritePanel when an IOWarrior in
        /// the device ListBox gets selected/unselected
        /// </summary>
        /// <param name="sender">
        /// We don't care who send this (don't do this at home kids)
        /// </param>
        /// <param name="e">
        /// We don't care about the args either (don't do this at home kids)
        /// </param>
        private void deviceListBox_SelectedIndexChanged(object sender, EventArgs e) {
            if(deviceListBox.SelectedItem==null) {
                writePanel1.setEnabled(false);
            }
            else {
                writePanel1.setEnabled(true);
            }
        }

        /// <summary>
        /// Called from the WritePanel through a delegate. This actually writes
        /// the data to the IOWarrior that is currently selected in the ListBox for devices.
        /// </summary>
        /// <param name="rep">
        /// The report we want to write.
        /// </param>
        public void writeToIow(IowReport rep) {
            IowError werr;
            //Get the IOWarrior that is currently selected in
            //the listbox for devices
            IOWarrior iow=deviceListBox.SelectedItem as IOWarrior;
            if(iow!=null) {
                //Write the report to the IOWarrior.
                werr=iow.write(rep);
                if(werr!=IowError.OK) {
                    //Ooops, the write failed, show a MessageBox
                    StringBuilder sb=new StringBuilder();
                    sb.Append("Write for device ");
                    sb.Append(iow.getProductName());
                    sb.Append(" ");
                    sb.Append(iow.getSerialNumberString());
                    sb.Append(" failed!");
                    sb.AppendLine();
                    sb.AppendLine(werr.getMessage());
                    MessageBox.Show(sb.ToString(), "Write Error");
                }
                else {
                    //Write was ok, add the data just written to
                    //the textbox for reports
                    writeUpdate(rep, iow);
                }
            }
        }
                    
    }
}